# from time import time

# class TextGraph():

#     def __init__(self):
#         pass

#     def execute(self):
#         t1 = time()

#         corpus_cleaned = f"{self.dataset_path}/{self.dataset_name}_clean.txt"

#         docs_of_words = [line.split() for line in open(file=corpus_cleaned)]
       
    